﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using System.Runtime.InteropServices;

namespace UsingTest
{
    public class MyDisposableObject : IDisposable
    {
        public static int NotDisposedCount = 0;
        private bool _disposed = false;
        ~MyDisposableObject()
        {
            if (!_disposed)
            {
                Interlocked.Increment(ref NotDisposedCount);
                Console.WriteLine("Not Disposed Object Found, {0} / {1} ({2} %)",
                    NotDisposedCount, Program.AbortCount, 100.0 * NotDisposedCount / Program.AbortCount);
            }
        }
        public void Dispose()
        {
            try
            {
            }
            finally
            {
                _disposed = true;
            }
        }
    }

    class Program
    {
        public static Queue<Thread> threadQueue = new Queue<Thread>();
        public static int AbortCount = 0;

        [DllImport("winmm.dll")]
        static extern uint timeBeginPeriod(uint period);
        [DllImport("winmm.dll")]
        static extern int timeEndPeriod(uint period);

        static void Main(string[] args)
        {
            uint? originalInterval = null; // invalid value for timeBeginPeriod
            try
            {
                //スレッド切り替えの粒度を細かくする
                originalInterval = timeBeginPeriod(1);

                Thread worker = new Thread(
                     delegate()
                     {
                         while (true)
                         {
                         	 Thread thread = null;
                             lock (threadQueue)
                             {
                                 if (threadQueue.Count > 1)
                                 {
                                     thread = threadQueue.Dequeue();
                                 }
                             }
                             if(thread != null)
                             {
                                 thread.Abort();
                                 Interlocked.Increment(ref AbortCount);
                             }
                             Thread.Sleep(50);
                         }
                     }
                );
                worker.Start();

                const int createThreadThreshold = 8;
                const int maxThread = 16;
                while (true)
                {
                    int threadQueueLength = 0;
                    lock (threadQueue)
                    {
                        threadQueueLength = threadQueue.Count;
                    }
                    if (threadQueueLength <= createThreadThreshold)
                    {
                        for (int i = threadQueueLength; i < maxThread; ++i)
                        {
                            Thread thread = null;
                            thread = new Thread(
                                delegate()
                                {
                                    lock (threadQueue)
                                    {
                                        threadQueue.Enqueue(thread);
                                    }
                                    while (true)
                                    {
                                        using (MyDisposableObject obj = new MyDisposableObject())
                                        {
                                        }
                                    }
                                }
                                );
                            thread.Priority = ThreadPriority.BelowNormal;
                            thread.Start();
                        }
                    }
                    if (threadQueueLength > createThreadThreshold)
                    {
                        Thread.Sleep(createThreadThreshold * 10);
                    }
                    if (AbortCount % 100 == 0)
                    {
                        GC.Collect();
                    }
                }
            }
            finally
            {
                if (originalInterval.HasValue)
                {
                    timeEndPeriod(originalInterval.Value);
                }
            }
        }
    }
}
